@@ -12,40 +12,15 @@ local cfg = require("luarocks.core.cfg")
 local repos = require("luarocks.repos")
 local writer = require("luarocks.manif.writer")
 
-local opts_mt = {}
-
-opts_mt.__index = opts_mt
-
-function opts_mt.type()
-   return "build.opts"
-end
-
-function build.opts(opts)
-   local valid_opts = {
-      need_to_fetch = "boolean",
-      minimal_mode = "boolean",
-      deps_mode = "string",
-      build_only_deps = "boolean",
-      namespace = "string?",
-      branch = "boolean",
-   }
-   for k, v in pairs(opts) do
-      local tv = type(v)
-      if not valid_opts[k] then
-         error("invalid build option: "..k)
-      end
-      local vo, optional = valid_opts[k]:match("^(.-)(%??)$")
-      if not (tv == vo or (optional == "?" and tv == nil)) then
-         error("invalid type build option: "..k.." - got "..tv..", expected "..vo)
-      end
-   end
-   for k, v in pairs(valid_opts) do
-      if (not v:find("?", 1, true)) and opts[k] == nil then
-         error("missing build option: "..k)
-      end
-   end
-   return setmetatable(opts, opts_mt)
-end
+build.opts = util.opts_table("build.opts", {
+   need_to_fetch = "boolean",
+   minimal_mode = "boolean",
+   deps_mode = "string",
+   build_only_deps = "boolean",
+   namespace = "string?",
+   branch = "string?",
+   verify = "boolean",
+})
 
 do
    --- Write to the current directory the contents of a table,
@@ -68,6 +43,16 @@ do
    function build.apply_patches(rockspec)
       assert(rockspec:type() == "rockspec")
 
+      if not (rockspec.build.extra_files or rockspec.build.patches) then
+         return true
+      end
+
+      local fd = io.open(fs.absolute_name(".luarocks.patches.applied"), "r")
+      if fd then
+         fd:close()
+         return true
+      end
+
       if rockspec.build.extra_files then
          extract_from_rockspec(rockspec.build.extra_files)
       end
@@ -82,6 +67,11 @@ do
             end
          end
       end
+
+      fd = io.open(fs.absolute_name(".luarocks.patches.applied"), "w")
+      if fd then
+         fd:close()
+      end
       return true
    end
 end
@@ -105,8 +95,8 @@ local function check_macosx_deployment_target(rockspec)
       if targetminor > versionminor then
          return nil, ("This rock requires Mac OSX 10.%d, and you are running 10.%d."):format(targetminor, versionminor)
       end
-      patch_variable("CC", target)
-      patch_variable("LD", target)
+      patch_variable("CC")
+      patch_variable("LD")
    end
    return true
 end
@@ -119,24 +109,31 @@ local function process_dependencies(rockspec, opts)
       end
    end
 
-   local ok, err, errcode = deps.check_lua(rockspec.variables)
+   local ok, err, errcode = deps.check_lua_incdir(rockspec.variables)
    if not ok then
       return nil, err, errcode
    end
 
+   if cfg.link_lua_explicitly then
+      local ok, err, errcode = deps.check_lua_libdir(rockspec.variables)
+      if not ok then
+         return nil, err, errcode
+      end
+   end
+
    if opts.deps_mode == "none" then
       util.warning("skipping dependency checks.")
       return true
    end
    if not opts.build_only_deps then
       if next(rockspec.build_dependencies) then
-         local ok, err, errcode = deps.fulfill_dependencies(rockspec, "build_dependencies", opts.deps_mode)
+         local ok, err, errcode = deps.fulfill_dependencies(rockspec, "build_dependencies", opts.deps_mode, opts.verify)
          if err then
             return nil, err, errcode
          end
       end
    end
-   local ok, err, errcode = deps.fulfill_dependencies(rockspec, "dependencies", opts.deps_mode)
+   local ok, err, errcode = deps.fulfill_dependencies(rockspec, "dependencies", opts.deps_mode, opts.verify)
    if err then
       return nil, err, errcode
    end
@@ -340,18 +337,7 @@ local function write_rock_dir_files(rockspec, opts)
 end
 
 --- Build and install a rock given a rockspec.
--- @param rockspec_file string: local or remote filename of a rockspec.
--- @param need_to_fetch boolean: true if sources need to be fetched,
--- false if the rockspec was obtained from inside a source rock.
--- @param minimal_mode boolean: true if there's no need to fetch,
--- unpack or change dir (this is used by "luarocks make"). Implies
--- need_to_fetch = false.
--- @param deps_mode string: Dependency mode: "one" for the current default tree,
--- "all" for all trees, "order" for all trees with priority >= the current default,
--- "none" for no trees.
--- @param build_only_deps boolean: true to build the listed dependencies only.
--- @param namespace string?: a namespace for the rockspec
--- @param branch string?: a forced branch to use
+-- @param opts table: build options table
 -- @return (string, string) or (nil, string, [string]): Name and version of
 -- installed rock if succeeded or nil and an error message followed by an error code.
 function build.build_rockspec(rockspec, opts)
@@ -399,10 +385,8 @@ function build.build_rockspec(rockspec, opts)
       fs.remove_dir_if_empty(path.versions_dir(name))
    end)
 
-   if not opts.minimal_mode then
-      local ok, err = build.apply_patches(rockspec)
-      if not ok then return nil, err end
-   end
+   ok, err = build.apply_patches(rockspec)
+   if not ok then return nil, err end
    
    ok, err = check_macosx_deployment_target(rockspec)
    if not ok then return nil, err end
