@@ -3,6 +3,7 @@
 -- Builds a rock, compiling its C parts if any.
 local cmd_build = {}
 
+local dir = require("luarocks.dir")
 local pack = require("luarocks.pack")
 local path = require("luarocks.path")
 local util = require("luarocks.util")
@@ -17,39 +18,29 @@ local search = require("luarocks.search")
 local make = require("luarocks.cmd.make")
 local cmd = require("luarocks.cmd")
 
-cmd_build.help_summary = "build/compile a rock."
-cmd_build.help_arguments = "[--pack-binary-rock] [--keep] {<rockspec>|<rock>|<name> [<version>]}"
-cmd_build.help = [[
-Build and install a rock, compiling its C parts if any.
-Argument may be a rockspec file, a source rock file
-or the name of a rock to be fetched from a repository.
+function cmd_build.add_to_parser(parser)
+   local cmd = parser:command("build", "Build and install a rock, compiling its C parts if any.\n"..
+      "If no arguments are given, behaves as luarocks make.", util.see_also())
+      :summary("Build/compile a rock.")
 
---pack-binary-rock  Do not install rock. Instead, produce a .rock file
-                    with the contents of compilation in the current
-                    directory.
+   cmd:argument("rock", "A rockspec file, a source rock file, or the name of "..
+      "a rock to be fetched from a repository.")
+      :args("?")
+   cmd:argument("version", "Rock version.")
+      :args("?")
 
---keep              Do not remove previously installed versions of the
-                    rock after building a new one. This behavior can
-                    be made permanent by setting keep_other_versions=true
-                    in the configuration file.
-
---branch=<name>     Override the `source.branch` field in the loaded
-                    rockspec. Allows to specify a different branch to 
-                    fetch. Particularly for "dev" rocks.
-
---only-deps         Installs only the dependencies of the rock.
-
-]]..util.deps_mode_help()
+   cmd:flag("--only-deps", "Installs only the dependencies of the rock.")
+   cmd:flag("--no-doc", "Installs the rock without its documentation.")
+   cmd:option("--branch", "Override the `source.branch` field in the loaded "..
+      "rockspec. Allows to specify a different branch to fetch. Particularly "..
+      'for "dev" rocks.')
+      :argname("<name>")
+   make.cmd_options(cmd)
+end
 
 --- Build and install a rock.
 -- @param rock_filename string: local or remote filename of a rock.
--- @param need_to_fetch boolean: true if sources need to be fetched,
--- false if the rockspec was obtained from inside a source rock.
--- @param deps_mode: string: Which trees to check dependencies for:
--- "one" for the current default tree, "all" for all trees,
--- "order" for all trees with priority >= the current default, "none" for no trees.
--- @param build_only_deps boolean: true to build the listed dependencies only.
--- @param namespace string?: an optional namespace
+-- @param opts table: build options
 -- @return boolean or (nil, string, [string]): True if build was successful,
 -- or false and an error message and an optional error code.
 local function build_rock(rock_filename, opts)
@@ -59,7 +50,7 @@ local function build_rock(rock_filename, opts)
    local ok, err, errcode
 
    local unpack_dir
-   unpack_dir, err, errcode = fetch.fetch_and_unpack_rock(rock_filename)
+   unpack_dir, err, errcode = fetch.fetch_and_unpack_rock(rock_filename, nil, opts.verify)
    if not unpack_dir then
       return nil, err, errcode
    end
@@ -90,7 +81,7 @@ local function do_build(ns_name, version, opts)
    if ns_name:match("%.rockspec$") or ns_name:match("%.rock$") then
       url = ns_name
    else
-      url, err = search.find_src_or_rockspec(ns_name, version)
+      url, err = search.find_src_or_rockspec(ns_name, version, true)
       if not url then
          return nil, err
       end
@@ -99,7 +90,7 @@ local function do_build(ns_name, version, opts)
    end
 
    if url:match("%.rockspec$") then
-      local rockspec, err, errcode = fetch.load_rockspec(url)
+      local rockspec, err, errcode = fetch.load_rockspec(url, nil, opts.verify)
       if not rockspec then
          return nil, err, errcode
       end
@@ -113,62 +104,83 @@ local function do_build(ns_name, version, opts)
    return build_rock(url, opts)
 end
 
+local function remove_doc_dir(name, version)
+   local install_dir = path.install_dir(name, version)
+   for _, f in ipairs(fs.list_dir(install_dir)) do
+      local doc_dirs = { "doc", "docs" }
+      for _, d in ipairs(doc_dirs) do
+         if f == d then
+            fs.delete(dir.path(install_dir, f))
+         end
+      end
+   end
+end
+
 --- Driver function for "build" command.
--- @param name string: A local or remote rockspec or rock file.
 -- If a package name is given, forwards the request to "search" and,
 -- if returned a result, installs the matching rock.
--- @param version string: When passing a package name, a version number may
--- also be given.
+-- When passing a package name, a version number may also be given.
 -- @return boolean or (nil, string, exitcode): True if build was successful; nil and an
 -- error message otherwise. exitcode is optionally returned.
-function cmd_build.command(flags, name, version)
-   assert(type(name) == "string" or not name)
-   assert(type(version) == "string" or not version)
-   
-   if not name then
-      return make.command(flags)
+function cmd_build.command(args)
+   if not args.rock then
+      return make.command(args)
    end
 
-   name = util.adjust_name_and_namespace(name, flags)
+   local name = util.adjust_name_and_namespace(args.rock, args)
 
    local opts = build.opts({
       need_to_fetch = true,
       minimal_mode = false,
-      deps_mode = deps.get_deps_mode(flags),
-      build_only_deps = not not flags["only-deps"],
-      namespace = flags["namespace"],
-      branch = not not flags["branch"],
+      deps_mode = deps.get_deps_mode(args),
+      build_only_deps = not not args.only_deps,
+      namespace = args.namespace,
+      branch = args.branch,
+      verify = not not args.verify,
    })
 
-   if flags["pack-binary-rock"] then
-      return pack.pack_binary_rock(name, version, function()
+   if args.sign and not args.pack_binary_rock then
+      return nil, "In the build command, --sign is meant to be used only with --pack-binary-rock"
+   end
+
+   if args.pack_binary_rock then
+      return pack.pack_binary_rock(name, args.version, args.sign, function()
          opts.build_only_deps = false
-         return do_build(name, version, opts)
+         local name, version, errcode = do_build(name, args.version, opts)
+         if name and args.no_doc then
+            remove_doc_dir(name, version)
+         end
+         return name, version, errcode
       end)
    end
    
-   local ok, err = fs.check_command_permissions(flags)
+   local ok, err = fs.check_command_permissions(args)
    if not ok then
       return nil, err, cmd.errorcodes.PERMISSIONDENIED
    end
 
-   ok, err = do_build(name, version, opts)
+   ok, err = do_build(name, args.version, opts)
    if not ok then return nil, err end
+   local version
    name, version = ok, err
 
+   if args.no_doc then
+      remove_doc_dir(name, version)
+   end
+
    if opts.build_only_deps then
       util.printout("Stopping after installing dependencies for " ..name.." "..version)
       util.printout()
    else
-      if (not flags["keep"]) and not cfg.keep_other_versions then
-         local ok, err = remove.remove_other_versions(name, version, flags["force"], flags["force-fast"])
+      if (not args.keep) and not cfg.keep_other_versions then
+         local ok, err = remove.remove_other_versions(name, version, args.force, args.force_fast)
          if not ok then
             util.printerr(err)
          end
       end
    end
 
-   writer.check_dependencies(nil, deps.get_deps_mode(flags))
+   writer.check_dependencies(nil, deps.get_deps_mode(args))
    return name, version
 end
 
