@@ -19,8 +19,8 @@ local sysdetect = require("luarocks.core.sysdetect")
 
 --------------------------------------------------------------------------------
 
-local program_version = "3.1.3"
-local program_series = "3.1"
+local program_version = "3.2.1"
+local program_series = "3.2"
 local major_version = (program_version:match("([^.]%.[^.])")) or program_series
 
 local is_windows = package.config:sub(1,1) == "\\"
@@ -356,6 +356,7 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
          lib = { "cyg?.dll", "?.dll", "lib?.dll" },
          include = { "?.h" }
       }
+      defaults.link_lua_explicitly = true
    end
 
    if platforms.unix then
@@ -370,6 +371,7 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
       defaults.variables.LD = "gcc"
       defaults.gcc_rpath = true
       defaults.variables.LIBFLAG = "-shared"
+      defaults.variables.TEST = "test"
 
       defaults.external_deps_patterns = {
          bin = { "?" },
@@ -480,43 +482,18 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
    return defaults
 end
 
-local function make_rocks_provided(lua_version, luajit_version)
-   local rocks_provided = {}
-   local rocks_provided_3_0 = {}
-
-   rocks_provided["lua"] = lua_version.."-1"
-
-   if lua_version == "5.2" or lua_version == "5.3" then
-      rocks_provided["bit32"] = lua_version.."-1"
-   end
-
-   if lua_version == "5.3" or lua_version == "5.4" then
-      rocks_provided["utf8"] = lua_version.."-1"
-   end
-
-   if luajit_version then
-      rocks_provided["luabitop"] = luajit_version.."-1"
-      rocks_provided_3_0["luajit"] = luajit_version.."-1"
-   end
-
-   return rocks_provided, rocks_provided_3_0
-end
-
 local function use_defaults(cfg, defaults)
 
-   -- Populate some arrays with values from their 'defaults' counterparts
+   -- Populate variables with values from their 'defaults' counterparts
    -- if they were not already set by user.
-   for _, entry in ipairs({"variables", "rocks_provided"}) do
-      if not cfg[entry] then
-         cfg[entry] = {}
-      end
-      for k,v in pairs(defaults[entry]) do
-         if not cfg[entry][k] then
-            cfg[entry][k] = v
-         end
+   if not cfg.variables then
+      cfg.variables = {}
+   end
+   for k,v in pairs(defaults.variables) do
+      if not cfg.variables[k] then
+         cfg.variables[k] = v
       end
    end
-   util.deep_merge_under(defaults.rocks_provided_3_0, cfg.rocks_provided)
 
    util.deep_merge_under(cfg, defaults)
 
@@ -536,10 +513,7 @@ local cfg = {}
 -- @param detected table containing information detected about the 
 -- environment. All fields below are optional:
 -- * lua_version (in x.y format, e.g. "5.3")
--- * luajit_version (complete, e.g. "2.1.0-beta3")
 -- * lua_bindir (e.g. "/usr/local/bin")
--- * lua_incdir (e.g. "/usr/local/include/lua5.3/")
--- * lua_libdir(e.g. "/usr/local/lib")
 -- * lua_dir (e.g. "/usr/local")
 -- * lua_interpreter (e.g. "lua-5.3")
 -- * project_dir (a string with the path of the project directory
@@ -553,15 +527,6 @@ function cfg.init(detected, warning)
    if not hc_ok then
       hardcoded = {}
    end
-
-   local lua_version = detected.lua_version or hardcoded.LUA_VERSION or _VERSION:sub(5)
-   local luajit_version = detected.luajit_version or hardcoded.LUAJIT_VERSION or (jit and jit.version:sub(8))
-   local lua_interpreter = detected.lua_interpreter or hardcoded.LUA_INTERPRETER or (arg and arg[-1] and arg[-1]:gsub(".*[\\/]", "")) or (is_windows and "lua.exe" or "lua")
-   local lua_bindir = detected.lua_bindir or hardcoded.LUA_BINDIR or (arg and arg[-1] and arg[-1]:gsub("[\\/][^\\/]+$", ""))
-   local lua_incdir = detected.lua_incdir or hardcoded.LUA_INCDIR
-   local lua_libdir = detected.lua_libdir or hardcoded.LUA_LIBDIR
-   local lua_dir = detected.lua_dir or hardcoded.LUA_DIR or (lua_bindir and lua_bindir:gsub("[\\/]bin$", ""))
-   local project_dir = detected.project_dir
    
    local init = cfg.init
 
@@ -577,16 +542,22 @@ function cfg.init(detected, warning)
    cfg.program_series = program_series
    cfg.major_version = major_version
 
-   cfg.lua_version = lua_version
-   cfg.luajit_version = luajit_version
-   cfg.lua_interpreter = lua_interpreter
+   -- Use detected values as defaults, overridable via config files or CLI args
 
-   cfg.variables = {
-      LUA_DIR = lua_dir,
-      LUA_BINDIR = lua_bindir,
-      LUA_INCDIR = lua_incdir,
-      LUA_LIBDIR = lua_libdir,
-   }
+   cfg.lua_version = detected.lua_version or hardcoded.LUA_VERSION or _VERSION:sub(5)
+   cfg.lua_interpreter = detected.lua_interpreter or hardcoded.LUA_INTERPRETER or (arg and arg[-1] and arg[-1]:gsub(".*[\\/]", "")) or (is_windows and "lua.exe" or "lua")
+   cfg.project_dir = (not hardcoded.FORCE_CONFIG) and detected.project_dir
+
+   do
+      local lua_bindir = detected.lua_bindir or hardcoded.LUA_BINDIR or (arg and arg[-1] and arg[-1]:gsub("[\\/][^\\/]+$", ""))
+      local lua_dir = detected.lua_dir or hardcoded.LUA_DIR or (lua_bindir and lua_bindir:gsub("[\\/]bin$", ""))
+      cfg.variables = {
+         LUA_DIR = lua_dir,
+         LUA_BINDIR = lua_bindir,
+         LUA_LIBDIR = hardcoded.LUA_LIBDIR,
+         LUA_INCDIR = hardcoded.LUA_INCDIR,
+      }
+   end
 
    cfg.init = init
 
@@ -637,8 +608,8 @@ function cfg.init(detected, warning)
       local name = "config-"..cfg.lua_version..".lua"
       sys_config_file = (cfg.sysconfdir .. "/" .. name):gsub("\\", "/")
       home_config_file = (cfg.homeconfdir .. "/" .. name):gsub("\\", "/")
-      if project_dir then
-         project_config_file = project_dir .. "/.luarocks/" .. name
+      if cfg.project_dir then
+         project_config_file = cfg.project_dir .. "/.luarocks/" .. name
       end
    end
 
@@ -652,7 +623,7 @@ function cfg.init(detected, warning)
    local home_config_ok
    local project_config_ok
    if not hardcoded.FORCE_CONFIG then
-      local env_var   = "LUAROCKS_CONFIG_" .. lua_version:gsub("%.", "_")
+      local env_var   = "LUAROCKS_CONFIG_" .. cfg.lua_version:gsub("%.", "_")
       local env_value = os.getenv(env_var)
       if not env_value then
          env_var   = "LUAROCKS_CONFIG"
@@ -681,7 +652,7 @@ function cfg.init(detected, warning)
       end
 
       -- finally, use the project-specific config file if any
-      if project_dir then
+      if cfg.project_dir then
          project_config_ok, err = load_config_file(cfg, platforms, project_config_file)
          if err then
             return nil, err, "config"
@@ -694,15 +665,16 @@ function cfg.init(detected, warning)
    -- Let's finish up the cfg table.
    ----------------------------------------
 
-   -- Settings detected or given via the CLI (i.e. --lua-dir) take precedence over config files:
-   cfg.project_dir = detected.project_dir
-   cfg.lua_version = detected.lua_version or cfg.lua_version
-   cfg.luajit_version = detected.luajit_version or cfg.luajit_version
-   cfg.lua_interpreter = detected.lua_interpreter or cfg.lua_interpreter
-   cfg.variables.LUA_BINDIR = detected.lua_bindir or cfg.variables.LUA_BINDIR or lua_bindir
-   cfg.variables.LUA_INCDIR = detected.lua_incdir or cfg.variables.LUA_INCDIR or lua_incdir
-   cfg.variables.LUA_LIBDIR = detected.lua_libdir or cfg.variables.LUA_LIBDIR or lua_libdir
-   cfg.variables.LUA_DIR = detected.lua_dir or cfg.variables.LUA_DIR or lua_dir
+   cfg.variables.LUA_DIR = detected.given_lua_dir or cfg.variables.LUA_DIR
+
+   -- Settings given via the CLI (i.e. --lua-dir) take precedence over config files.
+   cfg.project_dir = detected.given_project_dir or cfg.project_dir
+   cfg.lua_version = detected.given_lua_version or cfg.lua_version
+   if detected.given_lua_dir then
+      cfg.variables.LUA_DIR = detected.given_lua_dir
+      cfg.variables.LUA_BINDIR = detected.lua_bindir
+      cfg.lua_interpreter = detected.lua_interpreter
+   end
 
    -- Build a default list of rocks trees if not given
    if cfg.rocks_trees == nil then
@@ -715,7 +687,7 @@ function cfg.init(detected, warning)
       end
    end
 
-   local defaults = make_defaults(lua_version, processor, platforms, cfg.home)
+   local defaults = make_defaults(cfg.lua_version, processor, platforms, cfg.home)
 
    if platforms.windows and hardcoded.WIN_TOOLS then
       local tools = { "SEVENZ", "CP", "FIND", "LS", "MD5SUM", "PWD", "RMDIR", "WGET", "MKDIR" }
@@ -726,14 +698,13 @@ function cfg.init(detected, warning)
       defaults.fs_use_modules = true
    end
 
-   defaults.rocks_provided, defaults.rocks_provided_3_0 = make_rocks_provided(lua_version, luajit_version)
    use_defaults(cfg, defaults)
 
    cfg.variables.LUA = cfg.variables.LUA or (cfg.variables.LUA_BINDIR and (cfg.variables.LUA_BINDIR .. "/" .. cfg.lua_interpreter):gsub("//", "/"))
    cfg.user_agent = "LuaRocks/"..cfg.program_version.." "..cfg.arch
 
    cfg.config_files = {
-      project = project_dir and {
+      project = cfg.project_dir and {
          file = project_config_file,
          found = not not project_config_ok,
       },
@@ -751,6 +722,8 @@ function cfg.init(detected, warning)
                     and home_config_file
                     or sys_config_file),
    }
+   
+   cfg.cache = {}
 
    ----------------------------------------
    -- Attributes of cfg are set.
@@ -793,8 +766,8 @@ function cfg.init(detected, warning)
 
    function cfg.init_package_paths()
       local lr_path, lr_cpath, lr_bin = cfg.package_paths()
-      package.path = util.cleanup_path(package.path .. ";" .. lr_path, ";", lua_version)
-      package.cpath = util.cleanup_path(package.cpath .. ";" .. lr_cpath, ";", lua_version)
+      package.path = util.cleanup_path(package.path .. ";" .. lr_path, ";", cfg.lua_version, true)
+      package.cpath = util.cleanup_path(package.cpath .. ";" .. lr_cpath, ";", cfg.lua_version, true)
    end
 
    --- Check if platform was detected
@@ -805,12 +778,21 @@ function cfg.init(detected, warning)
       return platforms[name]
    end
 
-   function cfg.each_platform()
-      local i = 0
+   -- @param direction (optional) "least-specific-first" (default) or "most-specific-first"
+   function cfg.each_platform(direction)
+      direction = direction or "least-specific-first"
+      local i, delta
+      if direction == "least-specific-first" then
+         i = 0
+         delta = 1
+      else
+         i = #platform_order + 1
+         delta = -1
+      end
       return function()
          local p
          repeat
-            i = i + 1
+            i = i + delta
             p = platform_order[i]
          until (not p) or platforms[p]
          return p
