@@ -170,12 +170,13 @@ function deps.fulfill_dependency(dep, deps_mode, name, version, rocks_provided,
       return nil, "Could not satisfy dependency "..tostring(dep)..": "..search_err
    end
    util.printout("Installing "..url)
-   local install_flags = {
+   local install_args = {
+      rock = url,
       deps_mode = deps_mode,
       namespace = dep.namespace,
       verify = verify,
    }
-   local ok, install_err, errcode = install.command(install_flags, url)
+   local ok, install_err, errcode = install.command(install_args)
    if not ok then
       return nil, "Failed installing dependency: "..url.." - "..install_err, errcode
    end
@@ -497,7 +498,7 @@ function deps.scan_deps(results, manifest, name, version, deps_mode)
       rocks_provided = rockspec.rocks_provided
       mdn[version] = dependencies
    else
-      rocks_provided = setmetatable({}, { __index = cfg.rocks_provided_3_0 })
+      rocks_provided = util.get_rocks_provided()
    end
    local matched = deps.match_deps(dependencies, rocks_provided, nil, deps_mode)
    results[name] = version
@@ -531,12 +532,22 @@ local function find_lua_incdir(prefix, luaver, luajitver)
    return nil
 end
 
-function deps.check_lua(vars)
-   local incdir_found = true
+function deps.check_lua_incdir(vars)
+   local ljv = util.get_luajit_version()
+
    if (not vars.LUA_INCDIR) and vars.LUA_DIR then
-      vars.LUA_INCDIR = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, cfg.luajit_version)
-      incdir_found = (vars.LUA_INCDIR ~= nil)
+      vars.LUA_INCDIR = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, ljv)
+      if vars.LUA_INCDIR == nil then
+         return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency"
+      end
    end
+
+   return true
+end
+
+function deps.check_lua_libdir(vars)
+   local ljv = util.get_luajit_version()
+
    local shortv = cfg.lua_version:gsub("%.", "")
    local libnames = {
       "lua" .. cfg.lua_version,
@@ -545,7 +556,7 @@ function deps.check_lua(vars)
       "lua-" .. shortv,
       "lua",
    }
-   if cfg.luajit_version then
+   if ljv then
       table.insert(libnames, 1, "luajit-" .. cfg.lua_version)
    end
    local cache = {}
@@ -556,29 +567,11 @@ function deps.check_lua(vars)
          return true
       end
    end
-   if not incdir_found then
-      return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency"
-   end
    return nil, "Failed finding Lua library. You may need to configure LUA_LIBDIR.", "dependency"
 end
 
-local valid_deps_modes = {
-   one = true,
-   order = true,
-   all = true,
-   none = true,
-}
-
-function deps.check_deps_mode_flag(flag)
-   return valid_deps_modes[flag]
-end
-
-function deps.get_deps_mode(flags)
-   if flags["deps-mode"] then
-      return flags["deps-mode"]
-   else
-      return cfg.deps_mode
-   end
+function deps.get_deps_mode(args)
+   return args.deps_mode or cfg.deps_mode
 end
 
 return deps
